require 'aws/decider'

require 'fluere/config'
require 'fluere/exceptions'
require 'fluere/version'
require 'fluere/worker'
require 'fluere/workflow'

module Fluere
  def self.configure(&block)
    yield Fluere::Config
  end

  def self.workflows
    @workflows ||= []
  end

  def self.decisions_task_list
    Fluere.prefixed_task_list('decisions')
  end

  def self.activities_task_list
    Fluere.prefixed_task_list('activities')
  end

  def self.activities
    @activities ||= []
  end

  def self.swf
    @swf ||= AWS::SimpleWorkflow.new
  end

  def self.domain
    swf.domains[Config.domain]
  end

  def self.prefixed_task_list(task_list)
    [Fluere::Config.task_list_prefix, task_list].join('-')
  end

  def self.default_workflow_options
    {
      default_execution_start_to_close_timeout: 60 * 60 * 2,
      default_task_start_to_close_timeout: 60 * 60 * 2,
      default_child_policy: :terminate,
    }
  end

  def self.default_activity_options
    {
      default_task_schedule_to_close_timeout: 60 * 60 * 2,
      default_task_schedule_to_start_timeout: 60 * 60 * 2,
      default_task_start_to_close_timeout: 60 * 60 * 2,
    }
  end

  def self.start_execution(klass, *args)
    if stubbed?
      if perform_executions?
        info("Performing a stubbed execution, calling #{klass} decider block with args: #{args.inspect}")
        klass.decider_class.new.send(Fluere::Workflow::EXECUTION_METHOD, *args)
      else
        info("Skipped workflow execution for #{klass}")
      end
    else
      info("Requesting a workflow execution for #{klass}")
      klass.workflow_client.start_execution(*args)
    end
  end

  def self.info(msg)
    Fluere::Config.logger.info(msg)
  end

  def self.flush!
    domain.workflow_executions.with_status(:open).map { |we| we.terminate }
  end

  def self.stubbed?
    @stubbed
  end

  def self.stub!
    @stubbed = true
  end

  def self.unstub!
    @stubbed = false
  end

  def self.assert_stubbed
    raise StubbedPathOnlyError.new unless stubbed?
  end

  def self.perform_executions?
    !ignore_executions?
  end

  def self.ignore_executions?
    @ignore_executions
  end

  def self.ignore_executions
    raise "only allowed to ignore executions when stubbed!" unless stubbed?
    @ignore_executions = true
    yield
  ensure
    @ignore_executions = false
  end
end
